@@ -6,7 +6,7 @@ from enum import Enum
 from functools import total_ordering
 from operator import attrgetter
 from pathlib import Path
-from typing import List, Optional, Tuple, Union
+from typing import Any, Dict, List, Optional, Tuple, Union
 
 from fawltydeps.utils import hide_dataclass_fields
 
@@ -16,14 +16,15 @@ else:
     from typing_extensions import Literal
 
 SpecialPath = Literal["<stdin>"]
-PathOrSpecial = Union[Path, SpecialPath]
+PathOrSpecial = Union[SpecialPath, Path]
+TomlData = Dict[str, Any]  # type: ignore
 
 
-class ArgParseError(Exception):
-    """Indicate errors while parsing command-line arguments"""
+class UnparseablePathException(Exception):
+    """Exception type when alleged path (deps or code) can't be parsed"""
 
-    def __init__(self, msg: str):
-        self.msg = msg
+    def __init__(self, ctx: str, path: Path):
+        self.msg = f"{ctx}: {path}"
 
 
 class DependenciesMapping(Enum):
